
# a controller node through the OCI's ENC:
#
#---
#classes:
#   oci_controller:
#      is_first_master: true
#      first_master: zigo-controller-node-3.infomaniak.ch
#      first_master_ip: 192.168.100.40
#      vip_hostname: zigo-api.infomaniak.ch
#      vip_ipaddr: 192.168.101.2
#      network_ipaddr: 192.168.101.0
#      network_cidr: 24
#      other_masters:
#         - zigo-controller-node-4.infomaniak.ch
#         - zigo-controller-node-5.infomaniak.ch
#      other_masters_ip:
#         - 192.168.100.41
#         - 192.168.100.32
#      all_masters:
#         - zigo-controller-node-3.infomaniak.ch
#         - zigo-controller-node-4.infomaniak.ch
#         - zigo-controller-node-5.infomaniak.ch
#      all_masters_ip:
#         - 192.168.100.40
#         - 192.168.100.41
#         - 192.168.100.32
#
# This is re-used in the oci_controller class below
#
class oci::controller(
  $machine_hostname         = undef,
  $machine_ip               = undef,
  $is_first_master          = false,
  $first_master             = undef,
  $first_master_ip          = undef,
  $other_masters            = undef,
  $other_masters_ip         = undef,
  $vip_hostname             = undef,
  $vip_ipaddr               = undef,
  $vip_netmask              = undef,
  $swiftproxy_hostname      = 'none',
  $network_ipaddr           = undef,
  $network_cidr             = undef,
  $all_masters              = undef,
  $all_masters_ip           = undef,
  $all_masters_ids          = undef,
  $has_subrole_db           = true,
  $has_subrole_messaging    = true,
  $has_subrole_api_keystone = true,
  $has_subrole_heat         = true,
  $has_subrole_glance       = false,
  $has_subrole_nova         = false,
  $has_subrole_neutron      = false,
  $has_subrole_swift        = true,
  $has_subrole_horizon      = true,
  $has_subrole_barbican     = true,
  $has_subrole_cinder       = false,

  $glance_backend           = undef,
  $pass_glance_onswift      = undef,

  $haproxy_custom_url       = undef,
  $use_ssl                  = true,
  $rabbit_env               = {},
  $all_swiftproxy           = undef,
  $all_swiftproxy_ip        = undef,
  $pass_keystone_db         = undef,
  $pass_keystone_messaging  = undef,
  $pass_keystone_adminuser  = undef,
  $pass_keystone_credkey1   = undef,
  $pass_keystone_credkey2   = undef,
  $pass_keystone_fernkey1   = undef,
  $pass_keystone_fernkey2   = undef,
  $pass_nova_db             = undef,
  $pass_nova_apidb          = undef,
  $pass_nova_messaging      = undef,
  $pass_nova_authtoken      = undef,
  $pass_metadata_proxy_shared_secret = undef,
  $pass_placement_db        = undef,
  $pass_placement_authtoken = undef,
  $pass_glance_db           = undef,
  $pass_glance_messaging    = undef,
  $pass_glance_authtoken    = undef,
  $pass_cinder_db           = undef,
  $pass_cinder_messaging    = undef,
  $pass_cinder_authtoken    = undef,
  $pass_neutron_db          = undef,
  $pass_neutron_messaging   = undef,
  $pass_neutron_authtoken   = undef,
  $pass_heat_db             = undef,
  $pass_heat_messaging      = undef,
  $pass_heat_authtoken      = undef,
  $pass_heat_keystone_domain = undef,
  $pass_swift_authtoken     = undef,
  $pass_swift_encryption    = undef,
  $pass_horizon_secretkey   = undef,
  $pass_barbican_db         = undef,
  $pass_barbican_messaging  = undef,
  $pass_barbican_authtoken  = undef,

  $cluster_has_osds         = false,
  $cluster_has_mons         = false,

  $ceph_fsid                = undef,
  $ceph_bootstrap_osd_key   = undef,
  $ceph_admin_key           = undef,
  $ceph_openstack_key       = undef,
  $ceph_mon_key             = undef,
  $ceph_mon_initial_members = undef,
  $ceph_mon_host            = undef,
){

  if $use_ssl {
    $proto = 'https'
    $messaging_default_port = '5671'
    $messaging_notify_port = '5671'
    $api_port = 443
  } else {
    $proto = 'http'
    $messaging_default_port = '5672'
    $messaging_notify_port = '5672'
    $api_port = 80
  }
  $messaging_default_proto = 'rabbit'
  $messaging_notify_proto = 'rabbit'

  $base_url = "${proto}://${vip_hostname}"
  $keystone_auth_uri  = "${base_url}:${api_port}/identity"
  $keystone_admin_uri = "${base_url}:${api_port}/identity-admin"
  #$memcached_servers  = ["127.0.0.1:11211"]
  $memcached_string = join([join($all_masters_ip,':11211,'), ':11211'],'')
  $memcached_servers  = ["${memcached_string}"]

    ensure_resource('file', '/root/oci-openrc', {
      'ensure'  => 'present',
      'content' => "
export OS_PROJECT_DOMAIN_NAME='default'
export OS_USER_DOMAIN_NAME='default'
export OS_PROJECT_NAME='admin'
export OS_USERNAME='admin'
export OS_PASSWORD='${pass_keystone_adminuser}'
export OS_AUTH_URL='https://${vip_hostname}/identity-admin/v3'
export OS_IDENTITY_API_VERSION=3
export OS_IMAGE_API_VERSION=2
export OS_CACERT=/etc/ssl/certs/oci-pki-oci-ca-chain.pem
",
    })

  if $vip_netmask == 32 {
    $vip_iface = 'lo'
  } else {
    $vip_iface = 'eth0'
  }

  ################################
  ### Add a virtual IP address ###
  ################################
  class { 'corosync':
    authkey                  => '/var/lib/puppet/ssl/certs/ca.pem',
    bind_address             => $machine_ip,
    unicast_addresses        => $all_masters_ip,
    cluster_name             => 'mycluster',
    enable_secauth           => true,
    set_votequorum           => true,
    quorum_members           => $all_masters_ip,
    quorum_members_ids       => $all_masters_ids,
    log_stderr               => false,
    log_function_name        => true,
    syslog_priority          => 'debug',
    debug                    => true,
  }
  corosync::service { 'pacemaker':
    version => '0',
  }->
  cs_property { 'stonith-enabled':
    value   => 'false',
  }->
  cs_property { 'no-quorum-policy':
    value   => 'stop',
  }->
  cs_primitive { 'openstack-api-vip':
    primitive_class => 'ocf',
    primitive_type  => 'IPaddr2',
    provided_by     => 'heartbeat',
    parameters      => { 'ip' => $vip_ipaddr, 'cidr_netmask' => $vip_netmask, 'nic' => $vip_iface },
    operations      => { 'monitor' => { 'interval' => '10s' } },
    before          => Anchor['keystone::service::begin'],
  }

  # Drop all services on public IP address, just let https APIs

  # First, general definirion (could be in global site.pp)
  resources { "firewall":
    purge   => true
  }
  class { 'firewall': }

  # Define firewall rules
  firewall { '001 deny public access to http':
    proto       => tcp,
    action      => drop,
    destination => "${vip_ipaddr}/${vip_netmask}",
    dport       => 80,
  }->
  firewall { '002 deny public access to mysql':
    proto       => tcp,
    action      => drop,
    destination => "${vip_ipaddr}/${vip_netmask}",
    dport       => [3306, 4567],
  }->
  firewall { '003 deny public access to bgpd':
    proto       => tcp,
    action      => drop,
    destination => "${vip_ipaddr}/${vip_netmask}",
    dport       => 179,
  }->
  firewall { '004 deny public access to epmd and rabbitmq':
    proto       => tcp,
    action      => drop,
    destination => "${vip_ipaddr}/${vip_netmask}",
    dport       => [4369, 5671, 5672, 15671, 25672],
  }->
  firewall { '005 deny public access to rpcbind':
    proto       => tcp,
    action      => drop,
    destination => "${vip_ipaddr}/${vip_netmask}",
    dport       => [111],
  }->
  firewall { '006 deny public access to xinetd':
    proto       => tcp,
    action      => drop,
    destination => "${vip_ipaddr}/${vip_netmask}",
    dport       => [9200],
  }->
  firewall { '007 deny public access to horizon without haproxy':
    proto       => tcp,
    action      => drop,
    destination => "${vip_ipaddr}/${vip_netmask}",
    dport       => [7080, 7443],
  }

  # Add haproxy that will listen on that virtual IP, and load balance
  # to the different API daemons using tcp mode (as the APIs will do
  # full SSL already).

  # First, we accept binding on non-local IPs:
  sysctl::value { 'net.ipv4.ip_nonlocal_bind':
    value => "1",
    target => '/etc/sysctl.d/ip-nonlocal-bind.conf',
  }->
  # Then we setup HA proxy to bind on that IP
  if $use_ssl {
    file { "/etc/haproxy/ssl":
      ensure                  => directory,
      owner                   => 'root',
      mode                    => '0755',
      selinux_ignore_defaults => true,
      require       => Package['haproxy'],
    }->
    file { "/etc/haproxy/ssl/private":
      ensure                  => directory,
      owner                   => 'root',
      mode                    => '0755',
      selinux_ignore_defaults => true,
    }->
    file { "/etc/haproxy/ssl/private/oci-pki-api.pem":
      ensure                  => present,
      owner                   => "haproxy",
      source                  => "/etc/ssl/private/oci-pki-api.pem",
      selinux_ignore_defaults => true,
      mode                    => '0600',
    }
  }
  class { 'haproxy':
    global_options   => {
      'maxconn' => '40960',
      'user'    => 'haproxy',
      'group'   => 'haproxy',
      'daemon'  => '',
    },
    defaults_options => {
      'mode'    => 'http',
    },
    merge_options => true,
  }
# TODO: For security, we want this in the global section:
# global
#   ssl-default-bind-ciphers ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256
#   ssl-default-bind-options no-sslv3 no-tlsv10 no-tlsv11 no-tls-tickets
#   ssl-default-server-ciphers ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256
#   ssl-default-server-options no-sslv3 no-tlsv10 no-tlsv11 no-tls-tickets
# frontend ft_test
#   mode    http
#   bind    :443 ssl crt /path/to/<cert+privkey+intermediate+dhparam>
#   option forwardfor
#   http-request set-header X-Forwarded-Port %[dst_port]
#   http-request add-header X-Forwarded-Proto https if { ssl_fc }

  $haproxy_options_keystone = [
    { 'option'      => "forwardfor except $vip_ipaddr"},
    { 'acl'         => 'url_keystone_admin path_beg -i /identity-admin'},
    { 'use_backend' => 'keystoneadmbe if url_keystone_admin'},
    { 'acl'         => 'url_keystone path_beg -i /identity'},
    { 'use_backend' => 'keystonepubbe if url_keystone'},
  ]

  $glance_haproxy_options = [
    { 'acl'         => 'url_glance path_beg -i /image'},
    { 'use_backend' => 'glancebe if url_glance'},
  ]

  $swift_haproxy_options = [
    { 'acl'         => 'url_swift path_beg -i /object'},
    { 'use_backend' => 'swiftbe if url_swift'},
  ]

  $heat_haproxy_options = [
    { 'acl'         => 'url_heatcfn path_beg -i /orchestration-cfn'},
    { 'use_backend' => 'heatcfnbe if url_heatcfn'},
    { 'acl'         => 'url_heat path_beg -i /orchestration-api'},
    { 'use_backend' => 'heatbe if url_heat'},
  ]

  $horizon_haproxy_options = [
    { 'acl'         => 'url_horizon path_beg -i /horizon'},
    { 'use_backend' => 'horizonbe if url_horizon'},
  ]

  $barbican_haproxy_options = [
    { 'acl'         => 'url_barbican path_beg -i /keymanager'},
    { 'use_backend' => 'barbicanbe if url_barbican'},
  ]

  $nova_haproxy_options = [
    { 'acl'         => 'url_nova path_beg -i /compute'},
    { 'use_backend' => 'novabe if url_nova'},
    { 'acl'         => 'url_placement path_beg -i /placement'},
    { 'use_backend' => 'placementbe if url_placement'},
  ]

  $neutron_haproxy_options = [
    { 'acl'         => 'url_neutron path_beg -i /network'},
    { 'use_backend' => 'neutronbe if url_neutron'},
  ]

  $cinder_haproxy_options = [
    { 'acl'         => 'url_cinder path_beg -i /volume'},
    { 'use_backend' => 'cinderbe if url_cinder'},
  ]

  $custom_haproxy_options = [
    { 'acl'         => 'url_slash path /'},
    { 'redirect'    => "location $haproxy_custom_url if url_slash"},
  ]

  if $has_subrole_glance {
    $haproxy_options_with_glance = concat($haproxy_options_keystone, $glance_haproxy_options)
  } else {
    $haproxy_options_with_glance = $haproxy_options_keystone
  }

  if $has_subrole_swift and $swiftproxy_hostname == 'none' {
    $haproxy_options_with_swift = concat($haproxy_options_with_glance, $swift_haproxy_options)
  } else {
    $haproxy_options_with_swift = $haproxy_options_with_glance
  }

  if $has_subrole_heat {
    $haproxy_options_with_heat = concat($haproxy_options_with_swift, $heat_haproxy_options)
  } else {
    $haproxy_options_with_heat = $haproxy_options_with_swift
  }

  if $has_subrole_horizon {
    $haproxy_options_with_horizon = concat($haproxy_options_with_heat, $horizon_haproxy_options)
  } else {
    $haproxy_options_with_horizon = $haproxy_options_with_heat
  }

  if $has_subrole_barbican {
    $haproxy_options_with_barbican = concat($haproxy_options_with_horizon, $barbican_haproxy_options)
  } else {
    $haproxy_options_with_barbican = $haproxy_options_with_horizon
  }

  if $has_subrole_nova {
    $haproxy_options_with_nova = concat($haproxy_options_with_barbican, $nova_haproxy_options)
  } else {
    $haproxy_options_with_nova = $haproxy_options_with_barbican
  }

  if $has_subrole_neutron {
    $haproxy_options_with_neutron = concat($haproxy_options_with_nova, $neutron_haproxy_options)
  } else {
    $haproxy_options_with_neutron = $haproxy_options_with_nova
  }

  if $has_subrole_cinder {
    $haproxy_options_with_cinder = concat($haproxy_options_with_neutron, $cinder_haproxy_options)
  } else {
    $haproxy_options_with_cinder = $haproxy_options_with_neutron
  }

  if $haproxy_custom_url {
    $haproxy_options_with_custom = concat($haproxy_options_with_cinder, $custom_haproxy_options)
  } else {
    $haproxy_options_with_custom = $haproxy_options_with_cinder
  }

  # Global frontend for all services with dispatch depending on the URL
  haproxy::frontend { 'openstackfe':
    mode      => 'http',
    bind      => { "${vip_ipaddr}:${api_port}" => ['ssl', 'crt', '/etc/haproxy/ssl/private/oci-pki-api.pem', 'crt', '/etc/haproxy/ssl/private/'] },
    options   => $haproxy_options_with_custom,
  }

  # Keystone public
  haproxy::backend { 'keystonepubbe':
    options => [
       { 'option' => 'forwardfor' },
       { 'mode' => 'http' },
       { 'balance' => 'roundrobin' },
       { 'reqrep'  => '^([^\ ]*\ /)identity[/]?(.*)     \1\2'},
    ],
  }
  haproxy::balancermember { 'keystonepubbm':
    listening_service => 'keystonepubbe',
    ipaddresses       => $all_masters_ip,
    server_names      => $all_masters,
    ports             => 5000,
    options           => 'check ssl verify none',
  }
# TODO: haproxy::balancermember needs an option like this one:
#  ssl verify required ca-file /etc/haproxy/cert.d/key1.pem check

  # keystone admin
  haproxy::backend { 'keystoneadmbe':
    options => [
       { 'option' => 'forwardfor' },
       { 'mode' => 'http' },
       { 'balance' => 'roundrobin' },
       { 'reqrep'  => '^([^\ ]*\ /)identity-admin[/]?(.*)     \1\2'},
    ],
  }
  haproxy::balancermember { 'keystoneadmbm':
    listening_service => 'keystoneadmbe',
    ipaddresses       => $all_masters_ip,
    server_names      => $all_masters,
    ports             => 35357,
    options           => 'check ssl verify none',
  }

  # glance API
  if $has_subrole_glance {
    haproxy::backend { 'glancebe':
      options => [
         { 'option' => 'forwardfor' },
         { 'mode' => 'http' },
         { 'balance' => 'roundrobin' },
         { 'reqrep'  => '^([^\ ]*\ /)image[/]?(.*)     \1\2'},
      ],
    }
    haproxy::balancermember { 'glancebm':
      listening_service => 'glancebe',
      ipaddresses       => $all_masters_ip,
      server_names      => $all_masters,
      ports             => 9292,
      options           => 'check ssl verify none',
    }
  }

  # swift proxy
  if $has_subrole_swift and $swiftproxy_hostname == 'none' {
    haproxy::backend { 'swiftbe':
      options => [
         { 'option' => 'httpchk GET /healthcheck' },
         { 'option' => 'forwardfor' },
         { 'mode' => 'http' },
         { 'balance' => 'roundrobin' },
         { 'reqrep'  => '^([^\ ]*\ /)object[/]?(.*)     \1\2'},
      ],
    }
    haproxy::balancermember { 'swiftbm':
      listening_service => 'swiftbe',
      ipaddresses       => $all_swiftproxy_ip,
      server_names      => $all_swiftproxy,
      ports             => 8080,
      options           => 'check',
    }
  }

  # heat
  if $has_subrole_heat {
    haproxy::backend { 'heatbe':
      options => [
         { 'option' => 'forwardfor' },
         { 'mode' => 'http' },
         { 'balance' => 'roundrobin' },
         { 'reqrep'  => '^([^\ ]*\ /)orchestration-api[/]?(.*)     \1\2'},
      ],
    }
    haproxy::balancermember { 'heatbm':
      listening_service => 'heatbe',
      ipaddresses       => $all_masters_ip,
      server_names      => $all_masters,
      ports             => 8004,
      options           => 'check ssl verify none',
    }
    haproxy::backend { 'heatcfnbe':
      options => [
         { 'option' => 'forwardfor' },
         { 'mode' => 'http' },
         { 'balance' => 'roundrobin' },
         { 'reqrep'  => '^([^\ ]*\ /)orchestration-cfn[/]?(.*)     \1\2'},
      ],
    }
    haproxy::balancermember { 'heatcfnbm':
      listening_service => 'heatcfnbe',
      ipaddresses       => $all_masters_ip,
      server_names      => $all_masters,
      ports             => 8000,
      options           => 'check ssl verify none',
    }
  }

  if $has_subrole_horizon {
    haproxy::backend { 'horizonbe':
      options => [
         { 'option' => 'forwardfor' },
         { 'mode' => 'http' },
         { 'balance' => 'source' },
      ],
    }
    haproxy::balancermember { 'horizonbm':
      listening_service => 'horizonbe',
      ipaddresses       => $all_masters_ip,
      server_names      => $all_masters,
      ports             => 7443,
      options           => 'check ssl verify none',
    }
  }

  if $has_subrole_barbican {
    haproxy::backend { 'barbicanbe':
      options => [
         { 'option' => 'forwardfor' },
         { 'mode' => 'http' },
         { 'balance' => 'roundrobin' },
         { 'reqrep'  => '^([^\ ]*\ /)keymanager[/]?(.*)     \1\2'},
      ],
    }
    haproxy::balancermember { 'barbicanbm':
      listening_service => 'barbicanbe',
      ipaddresses       => $all_masters_ip,
      server_names      => $all_masters,
      ports             => 9311,
      options           => 'check ssl verify none',
    }
  }

  if $has_subrole_nova {
    haproxy::backend { 'novabe':
      options => [
         { 'option' => 'forwardfor' },
         { 'mode' => 'http' },
         { 'balance' => 'source' },
         { 'reqrep'  => '^([^\ ]*\ /)compute[/]?(.*)     \1\2'},
      ],
    }
    haproxy::balancermember { 'novabm':
      listening_service => 'novabe',
      ipaddresses       => $all_masters_ip,
      server_names      => $all_masters,
      ports             => 8774,
      options           => 'check ssl verify none',
    }

    haproxy::backend { 'placementbe':
      options => [
         { 'option' => 'forwardfor' },
         { 'mode' => 'http' },
         { 'balance' => 'roundrobin' },
         { 'reqrep'  => '^([^\ ]*\ /)placement[/]?(.*)     \1\2'},
      ],
    }
    haproxy::balancermember { 'placementbm':
      listening_service => 'placementbe',
      ipaddresses       => $all_masters_ip,
      server_names      => $all_masters,
      ports             => 8778,
      options           => 'check ssl verify none',
    }
  }

  if $has_subrole_neutron {
    haproxy::backend { 'neutronbe':
      options => [
         { 'option' => 'forwardfor' },
         { 'mode' => 'http' },
         { 'balance' => 'roundrobin' },
         { 'reqrep'  => '^([^\ ]*\ /)network[/]?(.*)     \1\2'},
      ],
    }
    haproxy::balancermember { 'neutronbm':
      listening_service => 'neutronbe',
      ipaddresses       => $all_masters_ip,
      server_names      => $all_masters,
      ports             => 9696,
      options           => 'check ssl verify none',
    }
  }

  if $has_subrole_cinder {
    haproxy::backend { 'cinderbe':
      options => [
         { 'option' => 'forwardfor' },
         { 'mode' => 'http' },
         { 'balance' => 'roundrobin' },
         { 'reqrep'  => '^([^\ ]*\ /)volume[/]?(.*)     \1\2'},
      ],
    }
    haproxy::balancermember { 'cinderbm':
      listening_service => 'cinderbe',
      ipaddresses       => $all_masters_ip,
      server_names      => $all_masters,
      ports             => 8776,
      options           => 'check ssl verify none',
    }
  }
  # Setup databases
  if $has_subrole_db {

    class { 'galera':
      galera_servers      => $all_masters_ip,
      galera_master       => $first_master,
      package_ensure      => 'latest',
      mysql_package_name  => 'mariadb-server',
      client_package_name => 'default-mysql-client',
      vendor_type         => 'mariadb',
      root_password       => 'a_big_secret',
      status_password     => 'a_big_secret',
      configure_repo      => false,
      configure_firewall  => false,
      galera_package_name => 'galera-3',
      override_options => {
        'mysqld' => {
          'bind_address'                    => '0.0.0.0',
          'wait_timeout'                    => '28800',
          'interactive_timeout'             => '30',
          'connect_timeout'                 => '30',
          'character_set_server'            => 'utf8',
          'collation_server'                => 'utf8_general_ci',
        }
      }
    }->
    class { '::keystone::db::mysql':
      dbname   => 'keystonedb',
      password => $pass_keystone_db,
        before => Anchor['keystone::service::begin'],
    }
    if $has_subrole_glance {
      class { '::glance::db::mysql':
        dbname   => 'glancedb',
        password => $pass_glance_db,
        before   => Anchor['glance::service::begin'],
      }
    }
    if $has_subrole_heat {
      class { '::heat::db::mysql':
        dbname   => 'heatdb',
        password => $pass_heat_db,
        before   => Anchor['heat::service::begin'],
      }
    }
    if $has_subrole_barbican {
      class { '::barbican::db::mysql':
        dbname   => 'barbicandb',
        password => $pass_barbican_db,
        before   => Anchor['barbican::service::begin'],
      }
    }
    if $has_subrole_nova {
      class { '::nova::db::mysql':
        user     => 'nova',
        dbname   => 'novadb',
        password => $pass_nova_db,
        before   => Class['::nova::cell_v2::simple_setup'],
        notify   => Service['nova-conductor', 'nova-scheduler'],
      }
      class { '::nova::db::mysql_api':
        user     => 'novaapi',
        dbname   => 'novaapidb',
        password => $pass_nova_apidb,
        before   => Class['::nova::cell_v2::simple_setup'],
      }
      class { '::nova::db::mysql_placement':
        user     => 'placement',
        dbname   => 'placementdb',
        password => $pass_placement_db,
        before   => Anchor['nova::service::begin'],
      }
    }
    if $has_subrole_neutron {
      class { '::neutron::db::mysql':
        dbname   => 'neutrondb',
        password => $pass_neutron_db,
        before   => Anchor['neutron::service::begin'],
      }
    }
    if $has_subrole_cinder {
      class { '::cinder::db::mysql':
        dbname        => 'cinderdb',
        password      => $pass_cinder_db,
        allowed_hosts => '%',
        before        => Anchor['cinder::service::begin'],
      }
    }
  }  

  # RabbitMQ
  if $has_subrole_messaging {
    if $use_ssl {
      file { "/etc/rabbitmq/ssl/private":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        require                 => File['/etc/rabbitmq/ssl'],
        selinux_ignore_defaults => true,
      }->
      file { "/etc/rabbitmq/ssl/public":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
      }->
      file { "/etc/rabbitmq/ssl/private/${::fqdn}.key":
        ensure                  => present,
        owner                   => "rabbitmq",
        source                  => "/etc/ssl/private/ssl-cert-snakeoil.key",
        selinux_ignore_defaults => true,
        mode                    => '0600',
      }->
      file { "/etc/rabbitmq/ssl/public/${::fqdn}.crt":
        ensure                  => present,
        owner                   => "rabbitmq",
        source                  => '/etc/ssl/certs/ssl-cert-snakeoil.pem',
        selinux_ignore_defaults => true,
        mode                    => '0644',
        notify        => Service['rabbitmq-server'],
      }
      $rabbit_ssl_cert = "/etc/rabbitmq/ssl/public/${::fqdn}.crt"
      $rabbit_ssl_key  = "/etc/rabbitmq/ssl/private/${::fqdn}.key"
      $rabbit_ssl_ca   = '/etc/ssl/certs/oci-pki-oci-ca-chain.pem'
    } else {
      $rabbit_ssl_cert = UNSET
      $rabbit_ssl_key  = UNSET
      $rabbit_ssl_ca   = UNSET
    }

    class { '::rabbitmq':
      package_ensure           => 'latest',
      delete_guest_user        => true,
      node_ip_address          => $machine_ip,
      ssl_interface            => $machine_ip,
      ssl                      => $use_ssl,
      ssl_only                 => $use_ssl,
      ssl_cacert               => $rabbit_ssl_ca,
      ssl_cert                 => $rabbit_ssl_cert,
      ssl_key                  => $rabbit_ssl_key,
      environment_variables    => $rabbit_env,
      repos_ensure             => false,
      # Clustering options...
      config_cluster           => true,
      cluster_nodes            => $all_masters,
      cluster_node_type        => 'ram',
      erlang_cookie            => 'XAFSRAFNNFJOFSHLEUFT',
      wipe_db_on_cookie_change => true,
    }->
    rabbitmq_vhost { '/':
      provider => 'rabbitmqctl',
      require  => Class['::rabbitmq'],
    }->
    rabbitmq_user { 'keystone':
      admin    => true,
      password => $pass_keystone_messaging,
    }->
    rabbitmq_user_permissions { "keystone@/":
      configure_permission => '.*',
      write_permission     => '.*',
      read_permission      => '.*',
      provider             => 'rabbitmqctl',
      require              => Class['::rabbitmq'],
    }

    if $has_subrole_glance {
      rabbitmq_user { 'glance':
        admin    => true,
        password => $pass_glance_messaging,
      }->
      rabbitmq_user_permissions { "glance@/":
        configure_permission => '.*',
        write_permission     => '.*',
        read_permission      => '.*',
        provider             => 'rabbitmqctl',
        require              => Class['::rabbitmq'],
        before               => Anchor['glance::service::begin'],
      }
    }

    if $has_subrole_heat {
      rabbitmq_user { 'heat':
        admin    => true,
        password => $pass_heat_messaging,
      }->
      rabbitmq_user_permissions { "heat@/":
        configure_permission => '.*',
        write_permission     => '.*',
        read_permission      => '.*',
        provider             => 'rabbitmqctl',
        require              => Class['::rabbitmq'],
        before               => Anchor['heat::service::begin'],
      }
    }

    if $has_subrole_barbican {
      rabbitmq_user { 'barbican':
        admin    => true,
        password => $pass_barbican_messaging,
      }->
      rabbitmq_user_permissions { "barbican@/":
        configure_permission => '.*',
        write_permission     => '.*',
        read_permission      => '.*',
        provider             => 'rabbitmqctl',
        require              => Class['::rabbitmq'],
        before               => Anchor['barbican::service::begin'],
      }
    }

    if $has_subrole_nova {
      rabbitmq_user { 'nova':
        admin    => true,
        password => $pass_nova_messaging,
      }->
      rabbitmq_user_permissions { "nova@/":
        configure_permission => '.*',
        write_permission     => '.*',
        read_permission      => '.*',
        provider             => 'rabbitmqctl',
        require              => Class['::rabbitmq'],
        before               => Anchor['nova::service::begin'],
      }
    }
    if $has_subrole_neutron {
      rabbitmq_user { 'neutron':
        admin    => true,
        password => $pass_neutron_messaging,
      }->
      rabbitmq_user_permissions { "neutron@/":
        configure_permission => '.*',
        write_permission     => '.*',
        read_permission      => '.*',
        provider             => 'rabbitmqctl',
        require              => Class['::rabbitmq'],
        before               => Anchor['neutron::service::begin'],
      }
    }
    if $has_subrole_cinder {
      rabbitmq_user { 'cinder':
        admin    => true,
        password => $pass_cinder_messaging,
      }->
      rabbitmq_user_permissions { "cinder@/":
        configure_permission => '.*',
        write_permission     => '.*',
        read_permission      => '.*',
        provider             => 'rabbitmqctl',
        require              => Class['::rabbitmq'],
        before               => Anchor['cinder::service::begin'],
      }
    }
  }

  # Get memcached
  class { '::memcached':
    listen_ip => $machine_ip,
    udp_port  => 0,
  }

  # Configure Apache to use mod-wsgi-py3, not mod-wsgi
  if $has_subrole_api_keystone {
    # Set python3 for mod-wsgi
    include ::apache::params
    class { '::apache':
      mod_packages => merge($::apache::params::mod_packages, {
        'wsgi' => 'libapache2-mod-wsgi-py3',
      })
    }
  }

  if $has_subrole_api_keystone {
    ##############################
    ### Setup keystone cluster ###
    ##############################
    class { '::keystone::client': }
    class { '::keystone::cron::token_flush': }

    if $use_ssl {
      file { "/etc/keystone/ssl":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
        require       => Package['keystone'],
      }->
      file { "/etc/keystone/ssl/private":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
      }->
      file { "/etc/keystone/ssl/public":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
      }->
      file { "/etc/keystone/ssl/private/${::fqdn}.key":
        ensure                  => present,
        owner                   => "keystone",
        source                  => "/etc/ssl/private/ssl-cert-snakeoil.key",
        selinux_ignore_defaults => true,
        mode                    => '0600',
      }->
      file { "/etc/keystone/ssl/public/${::fqdn}.crt":
        ensure                  => present,
        owner                   => "keystone",
        source                  => '/etc/ssl/certs/ssl-cert-snakeoil.pem',
        selinux_ignore_defaults => true,
        mode                    => '0644',
        notify        => Service['httpd'],
      }
    }

    ensure_resource('file', '/etc/apache2/sites-available/wsgi-keystone.conf', {
      'ensure'  => 'present',
      'content' => '',
      require   => Package['keystone'],
    })

    class { '::keystone':
      debug                      => true,
      database_connection        => "mysql://keystone:${pass_keystone_db}@127.0.0.1/keystonedb",
      catalog_type               => 'sql',
      admin_token                => $pass_keystone_adminuser,
      admin_password             => $pass_keystone_adminuser,
      enabled                    => true,
      service_name               => 'httpd',
      enable_ssl                 => $use_ssl,
      public_bind_host           => "${::fqdn}",
      admin_bind_host            => "${::fqdn}",
      manage_policyrcd           => true,
      enable_credential_setup    => true,
      credential_key_repository  => '/etc/keystone/credential-keys',
      credential_keys            => { '/etc/keystone/credential-keys/0' => { 'content' => $pass_keystone_credkey1 },
                                      '/etc/keystone/credential-keys/1' => { 'content' => $pass_keystone_credkey2 },
                                    },
      enable_fernet_setup        => true,
      fernet_key_repository      => '/etc/keystone/fernet-keys',
      fernet_max_active_keys     => 4,
      fernet_keys                => { '/etc/keystone/fernet-keys/0' => { 'content' => $pass_keystone_fernkey1 },
                                      '/etc/keystone/fernet-keys/1' => { 'content' => $pass_keystone_fernkey1 },
                                    },
      token_expiration           => 604800,
      admin_endpoint             => "https://${vip_hostname}:${api_port}/identity-admin",
      public_endpoint            => "https://${vip_hostname}:${api_port}/identity",
      default_transport_url      => os_transport_url({
        'transport' => 'rabbit',
        'hosts'     => $all_masters,
        'port'      => '5671',
        'username'  => 'keystone',
        'password'  => $pass_keystone_messaging,
      }),
      notification_transport_url => os_transport_url({
        'transport' => 'rabbit',
        'hosts'     => $all_masters,
        'port'      => '5671',
        'username'  => 'keystone',
        'password'  => $pass_keystone_messaging,
      }),
      rabbit_use_ssl             => $use_ssl,
      rabbit_ha_queues           => true,
      memcache_servers           => $memcached_servers,
    }
    include ::apache
    class { '::keystone::wsgi::apache':
      bind_host       => $machine_ip,
      admin_bind_host => $machine_ip,
      ssl             => $use_ssl,
      ssl_key         => "/etc/keystone/ssl/private/${::fqdn}.key",
      ssl_cert        => "/etc/keystone/ssl/public/${::fqdn}.crt",
      workers         => 2,
    }
    class { '::keystone::roles::admin':
      email    => 'production@infomaniak.com',
      password => $pass_keystone_adminuser,
    }->
    class { '::keystone::endpoint':
      public_url => "${base_url}/identity",
      admin_url  => "${base_url}/identity-admin",
    }
    class { '::keystone::disable_admin_token_auth': }
    class { '::openstack_extras::auth_file':
      password       => $pass_keystone_adminuser,
      project_domain => 'default',
      user_domain    => 'default',
      auth_url       => "${base_url}:${api_port}/identity/v3/",
    }
    keystone_role { 'creator':
      ensure => present,
    }
    keystone_role { 'member':
      ensure => present,
    }
  }

  #######################
  ### Setup Ceph keys ###
  #######################
  if $cluster_has_osds {
    $mon_keyring_path = "/tmp/ceph-${machine_hostname}.keyring"
    class { 'ceph':
      fsid                => $ceph_fsid,
      ensure              => 'present',
      authentication_type => 'cephx',
      mon_initial_members => $ceph_mon_initial_members,
      mon_host            => $ceph_mon_host,
    }->
    ceph::key { 'client.admin':
      secret  => $ceph_admin_key,
      cap_mon => 'allow *',
      cap_osd => 'allow *',
      cap_mds => 'allow',
    }->
    ceph::key { 'client.openstack':
      secret  => $ceph_openstack_key,
      mode    => '0644',
      cap_mon => 'profile rbd',
      cap_osd => 'profile rbd pool=cinder, profile rbd pool=nova, profile rbd pool=glance, profile rbd pool=gnocchi',
    }->
    ceph::key { 'client.bootstrap-osd':
      secret       => $ceph_bootstrap_osd_key,
      keyring_path => '/var/lib/ceph/bootstrap-osd/ceph.keyring',
      cap_mon      => 'allow profile bootstrap-osd',
    }

    if $cluster_has_osds {
      warning('Cluster has OSD nodes: will not setup MON and MGR on the controller nodes.')
    }else{
      exec { 'create-tmp-keyring':
        command => "/bin/true # comment to satisfy puppet syntax requirements
set -ex

cat > /etc/ceph/ceph.conf<< EOF
[global]
fsid = ${ceph_fsid}
mon_initial_members = ${ceph_mon_initial_members}
mon_host = ${ceph_mon_host}
auth cluster required = cephx
auth service required = cephx
auth client required = cephx
osd journal size = 1024
osd pool default size = 3
osd pool default min size = 2
osd pool default pg num = 333
osd pool default pgp num = 333
osd crush chooseleaf type = 1
EOF

cat > ${mon_keyring_path}<< EOF
[mon.]
    key = ${ceph_mon_key}
    caps mon = \"allow *\"
EOF

mon_data=\$(ceph-mon --cluster ceph --id ${machine_hostname} --show-config-value mon_data)

ceph-authtool ${mon_keyring_path} --import-keyring /etc/ceph/ceph.client.admin.keyring
ceph-authtool ${mon_keyring_path} --import-keyring /etc/ceph/ceph.client.openstack.keyring
ceph-authtool ${mon_keyring_path} --import-keyring /var/lib/ceph/bootstrap-osd/ceph.keyring
chmod 0444 ${mon_keyring_path}
chown ceph:ceph ${mon_keyring_path}

rm -f /tmp/monmap
monmaptool --create --add ${machine_hostname} ${machine_ip} --fsid ${ceph_fsid} /tmp/monmap
MACHINE_IPS=\$(echo ${ceph_mon_host} | tr ',' ' ')
CNT=1
for i in \$(echo ${ceph_mon_initial_members} | tr ',' ' ') ; do
        if [ \$i = ${machine_hostname} ] ; then
                echo -n ''
        else
                IP=\$(echo \$MACHINE_IPS | awk '{print \$'\$CNT'}')
                monmaptool --add \$i \$IP /tmp/monmap
        fi
        CNT=\$((\${CNT} + 1))
done

chown ceph:ceph /tmp/monmap

sudo -u ceph mkdir -p \$mon_data
sudo -u ceph ceph-mon --cluster ceph --mkfs -i ${machine_hostname} --monmap /tmp/monmap --keyring ${mon_keyring_path}
",
        unless  => "/bin/true # comment to satisfy puppet syntax requirements
set -ex
mon_data=\$(ceph-mon --cluster ceph --id ${machine_hostname} --show-config-value mon_data) || exit 1
# if ceph-mon fails then the mon is probably not configured yet
test -e \$mon_data/done
",
        require => Ceph::Key['client.bootstrap-osd'],
      }->
      ceph::mon { $machine_hostname:
        keyring     => $mon_keyring_path,
        public_addr => $machine_ip,
      }->
      ceph::mgr { $machine_hostname:
        key        => $ceph_mgr_key,
        inject_key => true,
      }
    }
    $ceph_pools = ['glance', 'nova', 'cinder', 'gnocchi']
    ceph::pool { $ceph_pools: }
  }

  ####################
  ### Setup Glance ###
  ####################
  if $has_subrole_glance {
    if $use_ssl {
      file { "/etc/glance/ssl":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
        require                 => Package['glance-api'],
      }->
      file { "/etc/glance/ssl/private":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
      }->
      file { "/etc/glance/ssl/public":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
      }->
      file { "/etc/glance/ssl/private/${::fqdn}.pem":
        ensure                  => present,
        owner                   => "glance",
        source                  => "/etc/ssl/private/ssl-cert-snakeoil.key",
        selinux_ignore_defaults => true,
        mode                    => '0600',
      }->
      file { "/etc/glance/ssl/public/${::fqdn}.crt":
        ensure                  => present,
        owner                   => "glance",
        source                  => '/etc/ssl/certs/ssl-cert-snakeoil.pem',
        selinux_ignore_defaults => true,
        mode                    => '0644',
        notify                  => Service['glance-api'],
      }

      $glance_key_file = "/etc/glance/ssl/private/${::fqdn}.pem"
      $glance_crt_file = "/etc/glance/ssl/public/${::fqdn}.crt"
    } else {
      $glance_key_file = undef
      $glance_crt_file = undef
    }

    include ::glance
    include ::glance::client
    if $is_first_master {
      class { '::glance::keystone::auth':
        public_url   => "${base_url}/image",
        internal_url => "${base_url}/image",
        admin_url    => "${base_url}/image",
        password     => $pass_glance_authtoken,
      }
    }

    class { '::glance::api::authtoken':
      password             => $pass_glance_authtoken,
      auth_url             => $keystone_admin_uri,
      www_authenticate_uri => $keystone_auth_uri,
      memcached_servers    => $memcached_servers,
      cafile               => '/etc/ssl/certs/oci-pki-oci-ca-chain.pem',
    }

    case $glance_backend {
      # This is there just in case, but doesn't really work, since
      # there's no mech to sync files between controller nodes.
      'file': {
        include ::glance::backend::file
        $backend_stores = ['file']
        $glance_default_store = 'file'
      }
      # This will be in use in priority, if OCI sees some OSD nodes.
      'ceph': {
        class { '::glance::backend::rbd':
          rbd_store_user => 'openstack',
          rbd_store_pool => 'glance',
        }
        $backend_stores = ['rbd']
        $glance_default_store = 'rbd'
        # make sure ceph pool exists before running Glance API
        Exec['create-glance'] -> Service['glance-api']
      }
      # Used if no OSD nodes are setup, but some swift store are present.
      'swift': {
        $backend_stores = ['swift']
        class { '::glance::backend::swift':
          swift_store_user                    => 'services:glance',
          swift_store_key                     => $pass_glance_onswift,
          swift_store_create_container_on_put => 'True',
          swift_store_auth_address            => "${keystone_auth_uri}/v3",
          swift_store_auth_version            => '3',
        }
        $glance_default_store = 'swift'
      }
    }

    class { '::glance::api':
      debug               => true,
      database_connection => "mysql+pymysql://glance:${pass_glance_db}@127.0.0.1/glancedb?charset=utf8",
      workers             => 2,
      stores              => $backend_stores,
      default_store       => $glance_default_store,
      bind_host           => $machine_ip,
      cert_file           => $glance_crt_file,
      key_file            => $glance_key_file,
      enable_v1_api       => false,
      enable_v2_api       => true,
    }

    class { '::glance::notify::rabbitmq':
      default_transport_url      => os_transport_url({
        'transport' => $messaging_default_proto,
        'hosts'     => $all_masters,
        'port'      => $messaging_default_port,
        'username'  => 'glance',
        'password'  => $pass_glance_messaging,
      }),
      notification_transport_url => os_transport_url({
        'transport' => $messaging_notify_proto,
        'hosts'     => $all_masters,
        'port'      => $messaging_notify_port,
        'username'  => 'glance',
        'password'  => $pass_glance_messaging,
      }),
      notification_driver        => 'messagingv2',
      rabbit_use_ssl             => $use_ssl,
    }
  }

  ###################
  ### Setup Swift ###
  ###################
  if $has_subrole_swift  {
    if $swiftproxy_hostname == 'none' {
      $swiftproxy_baseurl = $base_url
    } else {
      $swiftproxy_baseurl = "${proto}://${swiftproxy_hostname}"
    }
    class { '::swift::keystone::auth':
      public_url            => "${swiftproxy_baseurl}/object/v1/AUTH_%(tenant_id)s",
      admin_url             => "${swiftproxy_baseurl}/object",
      internal_url          => "${swiftproxy_baseurl}/object/v1/AUTH_%(tenant_id)s",
      password              => $pass_swift_authtoken,
      operator_roles        => ['admin', 'SwiftOperator', 'ResellerAdmin'],
      configure_s3_endpoint => false,
    }
  }

  ##################
  ### Setup Heat ###
  ##################
  if $has_subrole_heat {
    if $use_ssl {
      file { "/etc/heat/ssl":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
        require                 => Package['heat-api'],
      }->
      file { "/etc/heat/ssl/private":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
      }->
      file { "/etc/heat/ssl/public":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
      }->
      file { "/etc/heat/ssl/private/${::fqdn}.pem":
        ensure                  => present,
        owner                   => "heat",
        source                  => "/etc/ssl/private/ssl-cert-snakeoil.key",
        selinux_ignore_defaults => true,
        mode                    => '0600',
      }->
      file { "/etc/heat/ssl/public/${::fqdn}.crt":
        ensure                  => present,
        owner                   => "heat",
        source                  => '/etc/ssl/certs/ssl-cert-snakeoil.pem',
        selinux_ignore_defaults => true,
        mode                    => '0644',
        notify                  => Service['heat-api'],
      }

      $heat_key_file = "/etc/heat/ssl/private/${::fqdn}.pem"
      $heat_crt_file = "/etc/heat/ssl/public/${::fqdn}.crt"
    } else {
      $heat_key_file = undef
      $heat_crt_file = undef
    }
    class { '::heat::keystone::authtoken':
      password             => $pass_heat_authtoken,
      auth_url             => $keystone_admin_uri,
      www_authenticate_uri => $keystone_auth_uri,
      memcached_servers    => $memcached_servers,
      cafile               => '/etc/ssl/certs/oci-pki-oci-ca-chain.pem',
    }
    if $is_first_master {
      class { '::heat::keystone::auth':
        public_url   => "${base_url}/orchestration-api/v1/%(tenant_id)s",
        internal_url => "${base_url}/orchestration-api/v1/%(tenant_id)s",
        admin_url    => "${base_url}/orchestration-api/v1/%(tenant_id)s",
        password     => $pass_heat_authtoken,
      }
      class { '::heat::keystone::auth_cfn':
        public_url   => "${base_url}/orchestration-cfn/v1",
        internal_url => "${base_url}/orchestration-cfn/v1",
        admin_url    => "${base_url}/orchestration-cfn/v1",
        password     => $pass_heat_authtoken,
      }
    }

    class { '::heat':
      default_transport_url      => os_transport_url({
        'transport' => $messaging_default_proto,
        'hosts'     => $all_masters,
        'port'      => $messaging_default_port,
        'username'  => 'heat',
        'password'  => $pass_heat_messaging,
      }),
      notification_transport_url => os_transport_url({
        'transport' => $messaging_notify_proto,
        'hosts'     => $all_masters,
        'port'      => $messaging_notify_port,
        'username'  => 'heat',
        'password'  => $pass_heat_messaging,
      }),
      host                       => $machine_hostname,
      rabbit_use_ssl             => $use_ssl,
      database_connection        => "mysql+pymysql://heat:${pass_heat_db}@127.0.0.1/heatdb?charset=utf8",
      debug                      => true,
      notification_driver        => 'messagingv2',
    }
    heat_config {
      'clients_keystone/ca_file': value => '/etc/ssl/certs/oci-pki-oci-ca-chain.pem';
    }
    class { '::heat::keystone::domain':
      domain_password => $pass_heat_keystone_domain,
    }
    class { '::heat::client': }
    class { '::heat::api':
      service_name => 'httpd',
    }
    class { '::heat::wsgi::apache_api':
      bind_host       => $machine_ip,
      ssl             => $use_ssl,
      ssl_key         => $heat_key_file,
      ssl_cert        => $heat_crt_file,
      workers         => 2,
    }

    class { '::heat::engine':
      auth_encryption_key           => '1234567890AZERTYUIOPMLKJHGFDSQ12',
#      heat_metadata_server_url      => "${base_url}:8000/orchestration-cfn",
#      heat_waitcondition_server_url => "${base_url}:8000/orchestration-cfn/v1/waitcondition",
    }
    class { '::heat::api_cfn':
      service_name => 'httpd',
    }
    class { '::heat::wsgi::apache_api_cfn':
      bind_host => $machine_ip,
      ssl       => $use_ssl,
      ssl_key   => $heat_key_file,
      ssl_cert  => $heat_crt_file,
      workers   => 2,
    }
    class { '::heat::cron::purge_deleted': }

  }

  #####################
  ### Setup Horizon ###
  #####################
  if $has_subrole_horizon {
    if $use_ssl {
      file { "/etc/openstack-dashboard/ssl":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
        require                 => Class['::horizon'],
      }->
      file { "/etc/openstack-dashboard/ssl/private":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
      }->
      file { "/etc/openstack-dashboard/ssl/public":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
      }->
      file { "/etc/openstack-dashboard/ssl/private/${::fqdn}.pem":
        ensure                  => present,
        owner                   => "www-data",
        source                  => "/etc/ssl/private/ssl-cert-snakeoil.key",
        selinux_ignore_defaults => true,
        mode                    => '0600',
      }->
      file { "/etc/openstack-dashboard/ssl/public/${::fqdn}.crt":
        ensure                  => present,
        owner                   => "www-data",
        source                  => '/etc/ssl/certs/ssl-cert-snakeoil.pem',
        selinux_ignore_defaults => true,
        mode                    => '0644',
        notify                  => Service['httpd'],
      }

      $horizon_key_file = "/etc/openstack-dashboard/ssl/private/${::fqdn}.pem"
      $horizon_crt_file = "/etc/openstack-dashboard/ssl/public/${::fqdn}.crt"
    } else {
      $horizon_key_file = undef
      $horizon_crt_file = undef
    }
    class { '::horizon':
      secret_key       => $pass_horizon_secretkey,
      # TODO: Fix this with a more secure stuff, probably the FQDN of the API.
      allowed_hosts    => '*',
      listen_ssl       => true,
      ssl_redirect     => false,
      http_port       => '7080',
      https_port       => '7443',
      horizon_cert     => $horizon_crt_file,
      horizon_key      => $horizon_key_file,
      horizon_ca       => '/etc/ssl/certs/oci-pki-oci-ca-chain.pem',
      keystone_url     => "${keystone_auth_uri}/v3",
      compress_offline => true,
    }
  }
  ######################
  ### Setup Barbican ###
  ######################
  if $has_subrole_barbican {
    if $use_ssl {
      file { "/etc/barbican/ssl":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
        require                 => Anchor['barbican::install::end'],
      }->
      file { "/etc/barbican/ssl/private":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
      }->
      file { "/etc/barbican/ssl/public":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
      }->
      file { "/etc/barbican/ssl/private/${::fqdn}.pem":
        ensure                  => present,
        owner                   => "barbican",
        source                  => "/etc/ssl/private/ssl-cert-snakeoil.key",
        selinux_ignore_defaults => true,
        mode                    => '0600',
      }->
      file { "/etc/barbican/ssl/public/${::fqdn}.crt":
        ensure                  => present,
        owner                   => "barbican",
        source                  => '/etc/ssl/certs/ssl-cert-snakeoil.pem',
        selinux_ignore_defaults => true,
        mode                    => '0644',
        notify                  => Service['httpd'],
      }
      $barbican_key_file = "/etc/barbican/ssl/private/${::fqdn}.pem"
      $barbican_crt_file = "/etc/barbican/ssl/public/${::fqdn}.crt"
    } else {
      $barbican_key_file = undef
      $barbican_crt_file = undef
    }
    include ::barbican
    class { '::barbican::db':
      database_connection => "mysql+pymysql://barbican:${pass_barbican_db}@127.0.0.1/barbicandb?charset=utf8",
    }
    class { '::barbican::keystone::auth':
      public_url   => "${base_url}/keymanager",
      internal_url => "${base_url}/keymanager",
      admin_url    => "${base_url}/keymanager",
      password     => $pass_barbican_authtoken,
    }
    include ::barbican::quota
    include ::barbican::keystone::notification
    class { '::barbican::api::logging':
      debug => true,
    }
    class { '::barbican::keystone::authtoken':
      password             => $pass_barbican_authtoken,
      auth_url             => $keystone_admin_uri,
      www_authenticate_uri => $keystone_auth_uri,
      memcached_servers    => $memcached_servers,
      cafile               => '/etc/ssl/certs/oci-pki-oci-ca-chain.pem',
    }

    class { '::barbican::api':
      default_transport_url       => os_transport_url({
        'transport' => $messaging_default_proto,
        'hosts'     => $all_masters,
        'port'      => $messaging_default_port,
        'username'  => 'barbican',
        'password'  => $pass_barbican_messaging,
      }),
      notification_transport_url  => os_transport_url({
        'transport' => $messaging_default_proto,
        'hosts'     => $all_masters,
        'port'      => $messaging_default_port,
        'username'  => 'barbican',
        'password'  => $pass_barbican_messaging,
      }),
      host_href                   => "${base_url}/keymanager",
      auth_strategy               => 'keystone',
      service_name                => 'httpd',
      enabled_certificate_plugins => ['simple_certificate'],
      db_auto_create              => false,
      rabbit_use_ssl              => $use_ssl,
    }
    class { '::barbican::wsgi::apache':
      bind_host => $machine_ip,
      ssl       => $use_ssl,
      ssl_key   => $barbican_key_file,
      ssl_cert  => $barbican_crt_file,
      workers   => 2,
    }
#    ->
#    barbican_secret {'swift-barbican-secret':
#      ensure  => 'present',
#      payload => $pass_swift_encryption,
#      require => Class['::keystone'],
#    }
  }

  ##################
  ### Setup Nova ###
  ##################
  if $has_subrole_nova {
    if $use_ssl {
      file { "/etc/nova/ssl":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
        require                 => Anchor['nova::install::end'],
      }->
      file { "/etc/nova/ssl/private":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
      }->
      file { "/etc/nova/ssl/public":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
      }->
      file { "/etc/nova/ssl/private/${::fqdn}.pem":
        ensure                  => present,
        owner                   => "nova",
        source                  => "/etc/ssl/private/ssl-cert-snakeoil.key",
        selinux_ignore_defaults => true,
        mode                    => '0600',
      }->
      file { "/etc/nova/ssl/public/${::fqdn}.crt":
        ensure                  => present,
        owner                   => "nova",
        source                  => '/etc/ssl/certs/ssl-cert-snakeoil.pem',
        selinux_ignore_defaults => true,
        mode                    => '0644',
        notify                  => Service['httpd'],
      }
      $nova_key_file = "/etc/nova/ssl/private/${::fqdn}.pem"
      $nova_crt_file = "/etc/nova/ssl/public/${::fqdn}.crt"
    } else {
      $nova_key_file = undef
      $nova_crt_file = undef
    }
    include ::nova::cell_v2::simple_setup

    class { '::nova::keystone::auth':
      public_url   => "${base_url}/compute/v2.1",
      internal_url => "${base_url}/compute/v2.1",
      admin_url    => "${base_url}/compute/v2.1",
      password     => $pass_nova_authtoken,
    }

    class { '::nova::keystone::auth_placement':
      public_url   => "${base_url}/placement",
      internal_url => "${base_url}/placement",
      admin_url    => "${base_url}/placement",
      password     => $pass_placement_authtoken,
    }

    class { '::nova::keystone::authtoken':
      password             => $pass_nova_authtoken,
      auth_url             => $keystone_admin_uri,
      www_authenticate_uri => $keystone_auth_uri,
      memcached_servers    => $memcached_servers,
      cafile               => '/etc/ssl/certs/oci-pki-oci-ca-chain.pem',
    }

    class { '::nova::logging':
      debug => true,
    }

    class { '::nova':
      default_transport_url      => os_transport_url({
        'transport' => $messaging_default_proto,
        'hosts'     => $all_masters,
        'port'      => $messaging_default_port,
        'username'  => 'nova',
        'password'  => $pass_nova_messaging,
      }),
      notification_transport_url => os_transport_url({
        'transport' => $messaging_notify_proto,
        'hosts'     => $all_masters,
        'port'      => $messaging_notify_port,
        'username'  => 'nova',
        'password'  => $pass_nova_messaging,
      }),
      database_connection           => "mysql+pymysql://nova:${pass_nova_db}@127.0.0.1/novadb?charset=utf8",
      api_database_connection       => "mysql+pymysql://novaapi:${pass_nova_apidb}@127.0.0.1/novaapidb?charset=utf8",
      placement_database_connection => "mysql+pymysql://placement:${pass_placement_db}@127.0.0.1/placementdb?charset=utf8",
      rabbit_use_ssl                => $use_ssl,
      amqp_sasl_mechanisms          => 'PLAIN',
      use_ipv6                      => false,
      glance_api_servers            => "${base_url}/image",
      notification_driver           => 'messagingv2',
      notify_on_state_change        => 'vm_and_task_state',
      before                        => Class['::nova::cell_v2::simple_setup'],
    }

    class { '::nova::api':
      api_bind_address                     => $machine_ip,
      neutron_metadata_proxy_shared_secret => $pass_metadata_proxy_shared_secret,
      metadata_workers                     => 2,
      sync_db_api                          => true,
      service_name                         => 'httpd',
    }

    class { '::nova::wsgi::apache_api':
      bind_host => $machine_ip,
      ssl_key   => $nova_key_file,
      ssl_cert  => $nova_crt_file,
      ssl       => $use_ssl,
      workers   => '2',
    }

    class { '::nova::placement':
      auth_url => $keystone_admin_uri,
      password => $pass_placement_authtoken,
    }

    class { '::nova::client': }
    class { '::nova::conductor': }
    class { '::nova::consoleauth': }
    class { '::nova::cron::archive_deleted_rows': }

    class { '::nova::scheduler': }
    class { '::nova::scheduler::filter': }
    class { '::nova::vncproxy': }

    nova_config {
      'neutron/cafile':   value => '/etc/ssl/certs/oci-pki-oci-ca-chain.pem';
      'neutron/endpoint_override': value => "${base_url}/network";
      'glance/cafile':    value => '/etc/ssl/certs/oci-pki-oci-ca-chain.pem';
      'keystone/cafile':  value => '/etc/ssl/certs/oci-pki-oci-ca-chain.pem';
      'placement/cafile': value => '/etc/ssl/certs/oci-pki-oci-ca-chain.pem';
    }

    class { '::nova::network::neutron':
      neutron_auth_url      => "${keystone_auth_uri}/v3",
      neutron_url           => "${base_url}/network",
      neutron_password      => $pass_neutron_authtoken,
      default_floating_pool => 'public',
    }
  }
  #####################
  ### Setup Neutron ###
  #####################
  if $has_subrole_neutron {
    if $use_ssl {
      file { "/etc/neutron/ssl":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
        require                 => Anchor['neutron::install::end'],
      }->
      file { "/etc/neutron/ssl/private":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
      }->
      file { "/etc/neutron/ssl/public":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
      }->
      file { "/etc/neutron/ssl/private/${::fqdn}.pem":
        ensure                  => present,
        owner                   => "neutron",
        source                  => "/etc/ssl/private/ssl-cert-snakeoil.key",
        selinux_ignore_defaults => true,
        mode                    => '0600',
      }->
      file { "/etc/neutron/ssl/public/${::fqdn}.crt":
        ensure                  => present,
        owner                   => "neutron",
        source                  => '/etc/ssl/certs/ssl-cert-snakeoil.pem',
        selinux_ignore_defaults => true,
        mode                    => '0644',
        notify                  => Service['httpd'],
      }
      $neutron_key_file = "/etc/neutron/ssl/private/${::fqdn}.pem"
      $neutron_crt_file = "/etc/neutron/ssl/public/${::fqdn}.crt"
    } else {
      $neutron_key_file = undef
      $neutron_crt_file = undef
    }

    class { '::neutron::keystone::auth':
      public_url   => "${base_url}/network",
      internal_url => "${base_url}/network",
      admin_url    => "${base_url}/network",
      password     => $pass_neutron_authtoken,
    }

    class { '::neutron':
      default_transport_url      => os_transport_url({
        'transport' => $messaging_default_proto,
        'hosts'     => $all_masters,
        'port'      => $messaging_default_port,
        'username'  => 'neutron',
        'password'  => $pass_neutron_messaging,
      }),
      notification_transport_url => os_transport_url({
        'transport' => $messaging_notify_proto,
        'hosts'     => $all_masters,
        'port'      => $messaging_notify_port,
        'username'  => 'neutron',
        'password'  => $pass_neutron_messaging,
      }),
      rabbit_use_ssl             => $use_ssl,
      amqp_sasl_mechanisms       => 'PLAIN',
      allow_overlapping_ips      => true,
      core_plugin                => 'ml2',
      service_plugins            => ['router', 'metering', 'firewall', 'qos', 'trunk',],
      debug                      => true,
      bind_host                  => $machine_ip,
      use_ssl                    => $use_ssl,
      cert_file                  => $neutron_crt_file,
      key_file                   => $neutron_key_file,
      notification_driver        => 'messagingv2',
    }
    class { '::neutron::client': }
    class { '::neutron::keystone::authtoken':
      password             => $pass_neutron_authtoken,
      auth_url             => $keystone_admin_uri,
      www_authenticate_uri => $keystone_auth_uri,
      memcached_servers    => $memcached_servers,
      cafile               => '/etc/ssl/certs/oci-pki-oci-ca-chain.pem',
    }
    Service<| title == 'neutron-server'|> -> Openstacklib::Service_validation<| title == 'neutron-server' |> -> Neutron_network<||>
    class { '::neutron::server':
      database_connection => "mysql+pymysql://neutron:${pass_neutron_db}@127.0.0.1/neutrondb?charset=utf8",
      sync_db             => true,
      api_workers         => 2,
      rpc_workers         => 2,
      validate            => true,
      service_providers   => $providers_list,
    }

    neutron_config {
      'nova/cafile':   value => '/etc/ssl/certs/oci-pki-oci-ca-chain.pem';
    }

    class { '::neutron::services::lbaas': }

    class { '::neutron::plugins::ml2':
      type_drivers         => ['vxlan', 'vlan', 'flat'],
      tenant_network_types => ['vxlan', 'vlan', 'flat'],
      extension_drivers    => 'port_security,qos',
      mechanism_drivers    => 'openvswitch',
      firewall_driver      => 'iptables_hybrid',
    }
    class { '::neutron::agents::metadata':
      debug            => true,
      shared_secret    => $pass_metadata_proxy_shared_secret,
      metadata_workers => 2,
    }
    class { '::neutron::server::notifications':
      auth_url => $keystone_admin_uri,
      password => $pass_nova_authtoken,
    }
    class { '::neutron::services::fwaas':
      enabled       => true,
      agent_version => 'v1',
      driver        => 'neutron_fwaas.services.firewall.drivers.linux.iptables_fwaas.IptablesFwaasDriver',
    }
  }
  ####################
  ### Setup Cinder ###
  ####################
  if $has_subrole_cinder {
    if $use_ssl {
      file { "/etc/cinder/ssl":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
        require                 => Anchor['cinder::install::end'],
      }->
      file { "/etc/cinder/ssl/private":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
      }->
      file { "/etc/cinder/ssl/public":
        ensure                  => directory,
        owner                   => 'root',
        mode                    => '0755',
        selinux_ignore_defaults => true,
      }->
      file { "/etc/cinder/ssl/private/${::fqdn}.pem":
        ensure                  => present,
        owner                   => "cinder",
        source                  => "/etc/ssl/private/ssl-cert-snakeoil.key",
        selinux_ignore_defaults => true,
        mode                    => '0600',
      }->
      file { "/etc/cinder/ssl/public/${::fqdn}.crt":
        ensure                  => present,
        owner                   => "cinder",
        source                  => '/etc/ssl/certs/ssl-cert-snakeoil.pem',
        selinux_ignore_defaults => true,
        mode                    => '0644',
        notify                  => Service['httpd'],
      }
      $cinder_key_file = "/etc/cinder/ssl/private/${::fqdn}.pem"
      $cinder_crt_file = "/etc/cinder/ssl/public/${::fqdn}.crt"
    } else {
      $cinder_key_file = undef
      $cinder_crt_file = undef
    }

    # Cinder endpoints
    class { '::cinder::keystone::auth':
      public_url      => "${base_url}/volume/v1/%(tenant_id)s",
      internal_url    => "${base_url}/volume/v1/%(tenant_id)s",
      admin_url       => "${base_url}/volume/v1/%(tenant_id)s",
      public_url_v2   => "${base_url}/volume/v2/%(tenant_id)s",
      internal_url_v2 => "${base_url}/volume/v2/%(tenant_id)s",
      admin_url_v2    => "${base_url}/volume/v2/%(tenant_id)s",
      public_url_v3   => "${base_url}/volume/v3/%(tenant_id)s",
      internal_url_v3 => "${base_url}/volume/v3/%(tenant_id)s",
      admin_url_v3    => "${base_url}/volume/v3/%(tenant_id)s",
      password        => $pass_cinder_authtoken,
    }
    class { '::cinder':
      default_transport_url => os_transport_url({
        'transport' => $messaging_default_proto,
        'hosts'     => $all_masters,
        'port'      => $messaging_default_port,
        'username'  => 'cinder',
        'password'  => $pass_cinder_messaging,
      }),
      database_connection   => "mysql+pymysql://cinder:${pass_cinder_db}@127.0.0.1/cinderdb?charset=utf8",
      rabbit_use_ssl        => $use_ssl,
      debug                 => true,
    }
    class { '::cinder::keystone::authtoken':
      password             => $pass_cinder_authtoken,
      auth_url             => $keystone_admin_uri,
      www_authenticate_uri => $keystone_auth_uri,
      memcached_servers    => $memcached_servers,
      cafile               => '/etc/ssl/certs/oci-pki-oci-ca-chain.pem',
    }
    class { '::cinder::api':
      # Must match what we have in the volumes as enabled backend.
      default_volume_type        => 'LVM_1',
      public_endpoint            => "${base_url}/volume",
      service_name               => 'httpd',
      keymgr_backend             => 'castellan.key_manager.barbican_key_manager.BarbicanKeyManager',
      keymgr_encryption_api_url  => "${base_url}/keymanager",
      keymgr_encryption_auth_url => "${keystone_auth_uri}/v3",
    }
    class { '::cinder::wsgi::apache':
      bind_host => $machine_ip,
      ssl       => $use_ssl,
      ssl_key   => $cinder_key_file,
      ssl_cert  => $cinder_crt_file,
      workers   => 2,
    }
    class { '::cinder::quota': }
    class { '::cinder::scheduler': }
    class { '::cinder::scheduler::filter': }
    class { '::cinder::cron::db_purge': }
    class { '::cinder::glance':
      glance_api_servers => "${base_url}/image",
    }
  }
}
